///////////////////////////////////////////////////////////////////////////////
// BGStringTemplates.h
// Written by Bjoern Ganster since 2004
// Template versions of basic string functions
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include "BGBase.h"

#include <cstdint>

#define FILE_NUMBER FILE_NO_BGSTRINGTEMPLATES_H

///////////////////////////////////////////////////////////////////////////////
// bgstrlen, version with MaxChars

template<typename C>
size_t bgstrlen (const C* aStr, size_t MaxChars)
{
	SET_SCOPE;
	if (aStr == nullptr) {
		RETURN 0;
	}

	// Find end of aStr without exceeding MaxChars
	size_t i = 0;
	bool keepSearching = true;
	while (keepSearching) {
		if (i < MaxChars) {
			if (aStr[i] != 0) {
				++i;
			} else {
				keepSearching = false;
			}
		} else {
			keepSearching = false;
		}
	}

	RETURN i;
}

///////////////////////////////////////////////////////////////////////////////
// bgstrlen, version without MaxChars

template<typename C>
size_t bgstrlen (const C* a)
{
	SET_SCOPE;
	if (a != NULL) {
		size_t i = 0;

		// Find end of a 
		while (a[i] != 0) {
			i++;
		}

		RETURN i;
	} else { 
		RETURN 0;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Compare two strings - they may have different character size, but must
// use the same code point numbers

template<typename C, typename D>
int bgstrcmp (const C* a, const D* b)
{
	SET_SCOPE;
	if (a == (C*) b) {
      RETURN 0;
	}
	if (a == NULL && b == NULL) {
      RETURN 0;
	}
	if (a == NULL) {
      RETURN -1;
	}
	if (b == NULL) {
      RETURN 1;
	}

	// Find end of a 
	size_t i = 0;
	while (true) {
      if (a[i] == 0 || b[i] == 0 || a[i] != b[i]) {
         RETURN a[i]-b[i];
      }
      ++i;
	}

   RETURN 0;
}

///////////////////////////////////////////////////////////////////////////////
// Compare two strings, length-limited - strings may have different character 
// size, but must use the same code point numbers

template<typename C, typename D>
int bgstrcmpmax (const C* a, const D* b, size_t max)
{
	SET_SCOPE;
	if (a == NULL && b == NULL) {
		RETURN 0;
	}
	if (a == NULL) {
		RETURN -1;
	}
	if (b == NULL) {
		RETURN 1;
	}

	// Find end of a
	size_t i = 0;
	while (i < max) {
		if (a[i] == 0 || b[i] == 0 || a[i] != b[i]){
			RETURN a[i]-b[i];
		}
		++i;
	}

	RETURN 0;
}

///////////////////////////////////////////////////////////////////////////////
// bgstrlen, version with MaxChars

template<typename C>
size_t bgstrnlen (const C* a, size_t maxLen)
{
	SET_SCOPE;
	size_t i = 0;

	// Find end of a 
	while (a[i] != 0 && i != maxLen) {
		i++;
	}

	RETURN i;
}

///////////////////////////////////////////////////////////////////////////////
// bgstrlcpy

template<typename S, typename T>
void bgstrlcpy (T* target, const S* source, size_t MaxChars)
{
	SET_SCOPE;
	size_t i = 0;

	while (i < MaxChars-1 && source[i] != 0) {
		target[i] = (T) source[i];
		i++;
	}

	target[i] = 0;
	RETURN;
}

///////////////////////////////////////////////////////////////////////////////
// bgstrlcat

template<typename S, typename T>
void bgstrlcat (T* a, const S* b, size_t MaxChars)
{
	SET_SCOPE;
	size_t i = 0;
	size_t j = 0;

	// Find end of a 
	while (i < MaxChars-1 && a[i] != 0) {
		i++;
	}

	// Append b to a
	while (i+j < MaxChars-1 && b[j] != 0) {
		a[i+j] = b[j];
		j++;
	}

	// Null-Terminate result
	a[i+j] = 0;
	RETURN;
}

///////////////////////////////////////////////////////////////////////////////
// Scan a string for a character
// Find c with a search from start to stop, assuming start <= stop

template<typename C>
bool bgstrscan (const C* str, C c, size_t start, size_t stop, size_t& pos)
{
	SET_SCOPE;
	if (str == NULL) {
		RETURN false;
	}

	pos = start;

	while (str[pos] != c && pos < stop)
		pos++;

	bool result = false;
	if (pos < stop) {
		result = (str[pos] == c);
	}

	RETURN result;
}

///////////////////////////////////////////////////////////////////////////////
// Reverse scan for a character

template<typename C>
bool bgstrrscan (const C* str, C c, size_t start, size_t stop, size_t& pos)
{
	SET_SCOPE;
	if (str == NULL) {
		RETURN false;
	}

	SET_OP;
	pos = start;

	while (str[pos] != c && pos > stop)
		pos--;

	bool result = (str[pos] == c);
	RETURN result;
}

///////////////////////////////////////////////////////////////////////////////
// String copy with size limit

template<typename C, typename D>
void bgstrcpymax(C* target, const D* source, size_t maxcount)
{
	SET_SCOPE;
	if (target == NULL) {
		RETURN;
	}

	size_t i = 0;

	if (source != NULL) {
		while (i < maxcount && source[i] != 0) {
			target[i] = (C) (source[i]);
			++i;
		}
	}

	if (i < maxcount) 
		target[i] = 0;
	else if (maxcount > 0)
		target[maxcount-1] = 0;

	RETURN;
}


// Replace all occurrences of seekChar in str with replChar and return the number of replacements
template <typename C>
size_t replace (C* str, C seekChar, C replChar)
{
	SET_SCOPE;
	size_t replaced = 0;

	if (str != nullptr) {
		while (str[0] != 0) {
			if (str[0] == seekChar) {
				str[0] = replChar;
				++replaced;
			}
			++str;
		}
	}

	RETURN replaced;
}


// Check if str begins with substr
template<typename C>
bool beginsWith(const C* str, const C* substr)
{
	SET_SCOPE;
	if (substr == nullptr) {
		RETURN true ;
	}

	if (str == nullptr && substr[0] != 0) {
		RETURN false;
	}

	while (substr[0] != 0) {
		if (str[0] != substr[0]) {
			RETURN false;
		}
		++str;
		++substr;
	}

	RETURN true;
}

template<typename C>
int64_t bgstrToI(const C* str)
{
	SET_SCOPE;
	int64_t result = 0;
	bool negate = false;

	if (str != nullptr) {
		if (str[0] == '-') {
			negate = true;
			++str;
		}
		while (str[0] >= '0' && str[0] <= '9') {
			int cipher = str[0] - '0';
			result = 10 * result + cipher;
			++str;
		}
	}

	if (negate) {
		RETURN -result;
	} else {
		RETURN result;
	}
}

template<typename C>
double bgstrToD(const C* str)
{
	SET_SCOPE;
	double result = 0.0;
	bool negate = false;

	if (str != nullptr) {
		if (str[0] == '-') {
			negate = true;
			++str;
		}
		while (str[0] >= '0' && str[0] <= '9') {
			int cipher = str[0] - '0';
			result = 10 * result + cipher;
			++str;
		}

		if (str[0] == '.' || str[0] == ',') {
			++str;
			double factor = 0.1;
			while (str[0] >= '0' && str[0] <= '9') {
				int cipher = str[0] - '0';
				result += factor * cipher;
				factor *= 0.1;
				++str;
			}
		}
	}

	if (negate) {
		RETURN -result;
	} else {
		RETURN result;
	}
}

template<typename C>
C latinLowerCase(C c)
{
	SET_SCOPE;
	if (c >= 'A' && c <= 'Z') {
		RETURN c - 'A' + 'a';
	} else {
		RETURN c;
	}
}

template<typename C>
void appendInt(C* chars, int64_t num, int base, size_t maxCount)
{
	SET_SCOPE;
	if (num == 0) {
		bgstrlcat(chars, "0", maxCount);
	} else {
		int digits = 0;
		char buf[50];
		const char* digitChars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		if (num < 0) {
			buf[digits] = 0;
			++digits;
			num = -num;
		}

		// Build an array with the digits, reversed
		while (num != 0) {
			int digit = (num % base);
			num = num / base;
			buf[digits] = digitChars[digit];
			++digits;
		}

		size_t start = bgstrlen(chars, maxCount);
		if (start + digits < maxCount) {
			while (digits > 0) {
				--digits;
				chars[start+digits] = buf[digits];
			}
		}
	}

	RETURN;
}

#undef FILE_NUMBER
