// Interface for IO
// Written by Bjrn Ganster in 2019 based on older code for X-Ways Software Technology AG

#pragma once

#include <IL/BGBase.h>
#include <IL/il2.h>

class IIO {
public:
	virtual void close() = 0;
	virtual ILint64 read(void* buffer, ILuint size, ILuint number) = 0;

	virtual ILint64 seek(ILint64 offset, ILuint mode) = 0;
	virtual ILboolean eof() = 0;
	virtual ILint getc() = 0;
	virtual ILint64 tell() = 0;
	virtual ILint putc(ILubyte) = 0;
	virtual ILint64 write(const void* buffer, ILuint size, ILuint number) = 0;
};

#include <algorithm>

class FakeIO: public IIO {
public:
	FakeIO()
	: mOffset(0),
	  mSize(0)
	{
	}

	~FakeIO()
	{
	}

	void close() OVERRIDE
	{
	}

	ILint64 read(void* buffer, ILuint size, ILuint number) OVERRIDE
	{
		size_t sizeTotal = size * number;
		mOffset += sizeTotal;
		mSize = std::max(mOffset, mSize);
		return sizeTotal;
	}

	ILint64 seek(ILint64 offset, ILuint mode) OVERRIDE
	{
		ILint64 newOffset = mOffset;
		switch (mode) {
		case IL_SEEK_SET:
			newOffset = offset;
		case IL_SEEK_CUR:
			newOffset += offset;
			break;
		case IL_SEEK_END:
		case IL_EOF:
			newOffset = mSize - offset;
			break;
		}
		mOffset = (size_t) newOffset;

		if (mOffset == newOffset) {
			mSize = std::max(mOffset, mSize);
			return 0;
		}
		else {
			return 1; // error: seek beyond address space ...
		}
	}

	ILboolean eof() OVERRIDE
	{
		return (mOffset == mSize);
	}

	ILint getc() OVERRIDE
	{
		++mOffset;
		return 0;
	}

	ILint64 tell() OVERRIDE
	{
		return mOffset;
	}

	ILint putc(ILubyte c) OVERRIDE
	{
		++mOffset;
		return c;
	}

	ILint64 write(const void* buffer, ILuint size, ILuint number) OVERRIDE
	{
		size_t sizeTotal = size * number;
		mOffset += sizeTotal;
		mSize = std::max(mOffset, mSize);
		return sizeTotal;
	}

	size_t getSize() const 
	{
		return mSize;
	}
private:
	size_t mOffset;
	size_t mSize;
};
