//-----------------------------------------------------------------------------
//
// ResIL Sources
// Written by Bjrn Ganster in 2014
//
// Description: class for palette access
//
//-----------------------------------------------------------------------------

#ifndef IL_PALETTE_H
#define IL_PALETTE_H

#include <IL/BGBase.h>
//#include <IL/devil_internal_exports.h>
#include <IL/IIO.h>

#define FILE_NUMBER FILE_NO_IL_PALETTE_H

class ILpal FINAL
{
public:
	// Construct empty palette
	ILpal();

	// Copy palette
	ILpal(const ILpal& other);

	// Destructor
	~ILpal();

	// Use palette
	ILboolean use(ILint aNumCols, ILubyte* aPal, ILenum aPalType);

	ILboolean setRGBA(ILuint index, ILubyte red, ILubyte green, ILubyte blue, ILubyte alpha);

	ILboolean setRGB(ILuint index, ILubyte red, ILubyte green, ILubyte blue)
	{
		index *= mNumComponents;
		if (index + mNumComponents < mPalSize && mPalette != NULL) {
			mPalette[index + mRedOffset] = red;
			mPalette[index + mGreenOffset] = green;
			mPalette[index + mBlueOffset] = blue;
			return IL_TRUE;
		} else
			return IL_FALSE;
	}

	ILboolean getRGBA(ILuint index, ILubyte& red, ILubyte& green, ILubyte& blue, ILubyte& alpha)
	{
		index *= mNumComponents;
		if (index + mNumComponents < mPalSize 
		&&  mPalette != NULL) 
		{
			red   = mPalette[index + mRedOffset];
			green = mPalette[index + mGreenOffset];
			blue  = mPalette[index + mBlueOffset];
			if (mAlphaOffset >= 0) 
				alpha = mPalette[index + mAlphaOffset];
			return IL_TRUE;
		} else
			return IL_FALSE;
	}

	ILboolean getRGB(ILuint index, ILubyte& red, ILubyte& green, ILubyte& blue)
	{
		index *= mNumComponents;
		if (index + mNumComponents < mPalSize && mPalette != NULL) {
			red   = mPalette[index + mRedOffset];
			green = mPalette[index + mGreenOffset];
			blue  = mPalette[index + mBlueOffset];
			return IL_TRUE;
		} else
			return IL_FALSE;
	}

	void clear();

	ILenum getPalType() const
	{
		return mPalType;
	}

	ILuint getPalSize() const
	{
		return mPalSize;
	}

	ILboolean readFromFile(IIO* io)
	{
		auto read = io->read(mPalette, 1, mPalSize);
		if (read == mPalSize)
			return IL_TRUE;
		else
			return IL_FALSE;
	}

	ILboolean writeToFile(IIO* io)
	{
		auto written = io->write(mPalette, sizeof(ILubyte), mPalSize);
		if (written == mPalSize)
			return IL_TRUE;
		else
			return IL_FALSE;
	}

	ILboolean hasPalette() const
	{
		if (mPalette != NULL && mPalSize > 0 && mPalType != IL_PAL_NONE)
			return IL_TRUE;
		else
			return IL_FALSE;
	}

	void operator=(const ILpal& other)
	{
		if (other.hasPalette()) {
			use(other.mNumCols, other.mPalette, other.mPalType);
		} else
			clear();
	}

	ILuint getNumCols() const
	{
		return mNumCols;
	}

	// Return a pointer to the internal palette - USE CAUTIOUSLY!
	// The returned pointer should not be used to manipulate the palette
	ILubyte* getPalette()
	{
		return mPalette;
	}
private:
	ILubyte* mPalette; //!< the image palette (if any)
	ILuint   mPalSize; //!< size of the palette (in bytes)
	ILenum   mPalType; //!< the palette types in il.h (0x0500 range)
	ILuint   mNumCols; //!< number of colors in the palette, 
	ILbyte  mNumComponents, mRedOffset, mGreenOffset, mBlueOffset, mAlphaOffset; // negative values indicate unused components
};

#undef FILE_NUMBER

#endif