// LumpIO.cpp
// Implement IIO with pointer to data (called a Lump)
// Written by Bjrn Ganster in 2019 based on older code for X-Ways Software Technology AG


#include "LumpIO.h"

#include <IL/devil_internal_exports.h>

// Clear contents
void ConstLumpIO::clear()
{
	if (mDeleteData) {
		ifree(mData);
		mData = NULL;
		mSize = 0;
		mOffset = 0;
		mDeleteData = false;
	}
}

// Read lump from file
ILboolean ConstLumpIO::open(ILconst_string fileName) 
{
	clear();

#ifdef _WIN32
	Win32IO io2;
#else
	PosixIO io2;
#endif
	ILboolean result = IL_FALSE;
	if (io2.open(fileName)) {
		mSize = io2.getFileSize();
		void* temp = (unsigned char*)ialloc(mSize);
		if (temp != NULL) {
			auto read = io2.read((void*)temp, mSize, 1);
			mData = (const unsigned char*)temp;
			result = (read == mSize);
		}
		else {
			mSize = 0;
		}
	}
	return result;
}

// Read file contents into memory for updating
ILboolean WritableLumpIO::open(ILconst_string fileName) 
{
	clear();

	PosixIO pio;
	ILboolean result = IL_FALSE;
	if (pio.open(fileName)) {
		mSize = pio.getFileSize();
		void* temp = (unsigned char*)ialloc(mSize);
		if (mData != NULL) {
			auto read = pio.read((void*)mData, mSize, 1);
			mData = (unsigned char*)temp;
			result = (read == mSize);
		}
		else {
			mSize = 0;
		}
	}
	return result;
}

void WritableLumpIO::clear()
{
	if (mDeleteData) {
		ifree(mData);
		mData = NULL;
		mSize = 0;
		mOffset = 0;
		mDeleteData = false;
	}
}
