// Implement IIO with Posix file functions
// Written by Bjrn Ganster in 2019 based on older code for X-Ways Software Technology AG

#pragma once

#include <IL/IIO.h>

class PosixIO : public IIO {
public:
	PosixIO(FILE* handle = NULL, bool closeHandle = true)
		: mHandle(handle),
		mCloseHandle(closeHandle)
	{
	}

	~PosixIO() 
	{
		if (mCloseHandle) {
			close();
		}
	}

	ILboolean open(ILconst_string FileName)
	{
#ifndef _UNICODE
		mHandle = fopen((char*)FileName, "rb");
		return (mHandle != NULL);
#else
		// Windows has a different function, _wfopen, to open UTF16 files,
		// whereas Linux just uses fopen for its UTF8 files.
#ifdef _WIN32
		 int err =_wfopen_s(&mHandle, FileName, L"rb");
		 return (err == 0);
#else
		mHandle = fopen((char*)FileName, "rb");
		return (mHandle != NULL);
#endif
#endif//UNICODE

	}


	ILboolean ILAPIENTRY openWrite(ILconst_string FileName)
	{
#ifndef _UNICODE
		mHandle = fopen((char*)FileName, "wb");
		return (mHandle != NULL);
#else
		// Windows has a different function, _wfopen, to open UTF16 files,
		//  whereas Linux just uses fopen.
#ifdef _WIN32
		int err = _wfopen_s(&mHandle, FileName, L"wb");
		return (err == 0);
#else
		mHandle = fopen((char*)FileName, "wb");
		return (mHandle != NULL);
#endif
#endif//UNICODE
	}


	void close() OVERRIDE
	{
		if (mHandle != 0) {
			fclose(mHandle);
			mHandle = NULL;
		}
	}


	ILint64 read(void* buffer, ILuint size, ILuint number) OVERRIDE
	{
		return (ILint)fread(buffer, size, number, mHandle);
	}


	ILint64 seek(ILint64 offset, ILuint mode) OVERRIDE
	{
		long offsetLong = (long)offset;
		if (offsetLong != offset)
			return 1; // value too large

		switch (mode) {
		case IL_SEEK_SET:
			return fseek(mHandle, offsetLong, SEEK_SET);
		case IL_SEEK_CUR:
			return fseek(mHandle, offsetLong, SEEK_CUR);
		case IL_SEEK_END:
		case IL_EOF:
			return fseek(mHandle, offsetLong, SEEK_END);
		default:
			return 0;
		}
	}

	ILint64 getFileSize()
	{
		// Find out the filesize for checking for the end of file
		auto OrigPos = tell();
		seek(0, IL_SEEK_END);
		auto FileSize = tell();
		seek(OrigPos, IL_SEEK_SET);
		return FileSize;
	}

	ILboolean eof() OVERRIDE
	{
		auto FileSize = getFileSize();

		if (tell() >= FileSize)
			return IL_TRUE;
		else
			return IL_FALSE;
	}

	ILint getc() OVERRIDE
	{
		ILint Val;

		Val = 0;
		if (read(&Val, 1, 1) != 1)
			return IL_EOF;
		return Val;
	}

	ILint64 tell() OVERRIDE
	{
		return ftell(mHandle);
	}

	ILint putc(ILubyte Char) OVERRIDE
	{
		return fputc(Char, mHandle);
	}

	ILint64 write(const void* buffer, ILuint size, ILuint number) OVERRIDE
	{
		return (ILint)fwrite(buffer, size, number, mHandle);
	}
private:
	FILE* mHandle;
	boolean mCloseHandle;
};
