// Implement IIO with Win32 file functions
// Written by Bjrn Ganster in 2019 based on older code for X-Ways Software Technology AG

#pragma once

#ifdef _WIN32

#include <IL/IIO.h>

#include <Windows.h>

class Win32IO : public IIO {
public:
	Win32IO(HANDLE handle = INVALID_HANDLE_VALUE, bool closeHandle = true)
	: mHandle(handle),
	  mCloseHandle(closeHandle)
	{
	}

	~Win32IO() 
	{
		if (mCloseHandle) {
			close();
		}
	}

	ILboolean open(const char* fileName)
	{
		mHandle = CreateFileA(fileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		return (mHandle != INVALID_HANDLE_VALUE);
	}
	ILboolean open(const wchar_t* fileName)
	{
		mHandle = CreateFileW(fileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		return (mHandle != INVALID_HANDLE_VALUE);
	}

	ILboolean ILAPIENTRY openWrite(const char* fileName)
	{
		mHandle = CreateFileA(fileName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		return (mHandle != INVALID_HANDLE_VALUE);
	}
	ILboolean ILAPIENTRY openWrite(const wchar_t* fileName)
	{
		mHandle = CreateFileW(fileName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		return (mHandle != INVALID_HANDLE_VALUE);
	}

	void close() OVERRIDE
	{
		if (mHandle != INVALID_HANDLE_VALUE) {
			CloseHandle(mHandle);
			mHandle = INVALID_HANDLE_VALUE;
		}
	}

	ILint64 read(void* buffer, ILuint size, ILuint number) OVERRIDE
	{
		DWORD totalToRead = size * number;
		DWORD read = 0;
		ReadFile(mHandle, buffer, totalToRead, &read, NULL);
		return read;
	}

	ILint64 seek(ILint64 offset, ILuint mode) OVERRIDE
	{
		switch (mode) {
		case IL_SEEK_SET:
			return SetFilePointerEx(mHandle, *(LARGE_INTEGER*) (&offset), NULL, FILE_BEGIN);
		case IL_SEEK_CUR:
			return SetFilePointerEx(mHandle, *(LARGE_INTEGER*)(&offset), NULL, FILE_CURRENT);
		case IL_SEEK_END:
		case IL_EOF:
			return SetFilePointerEx(mHandle, *(LARGE_INTEGER*)(&offset), NULL, FILE_END);
		default:
			return 0;
		}
	}

	ILint64 getFileSize()
	{
		ILint64 size = 0;
		if (!GetFileSizeEx(mHandle, (LARGE_INTEGER*)(&size)))
			size = 0;
		return size;
	}

	ILboolean eof() OVERRIDE
	{
		auto FileSize = getFileSize();

		if (tell() >= FileSize)
			return IL_TRUE;
		else
			return IL_FALSE;
	}

	ILint getc() OVERRIDE
	{
		ILint Val;

		Val = 0;
		if (read(&Val, 1, 1) != 1)
			return IL_EOF;
		return Val;
	}

	ILint64 tell() OVERRIDE
	{
		ILint64 offset = 0;
		LARGE_INTEGER  liDistanceToMove;
		liDistanceToMove.QuadPart = 0;
		if (SetFilePointerEx(mHandle, liDistanceToMove, (LARGE_INTEGER*)(&offset), FILE_BEGIN))
			return offset;
		else
			return -1;
	}

	ILint putc(ILubyte c) OVERRIDE
	{
		return (ILint)write(&c, 1, 1);
	}

	ILint64 write(const void* buffer, ILuint size, ILuint number) OVERRIDE
	{
		DWORD totalToWrite = size * number;
		DWORD written = 0;
		WriteFile(mHandle, buffer, totalToWrite, &written, NULL);
		return written;
	}
private:
	HANDLE mHandle;
	boolean mCloseHandle;
};

#endif // _WIN32