//-----------------------------------------------------------------------------
//
// ImageLib Sources
// Copyright (C) 2000-2008 by Denton Woods
// Last modified: 06/02/2007
//
// Filename: src-IL/src/il_error.c
//
// Description: The error functions
//
//-----------------------------------------------------------------------------


#include "il_internal.h"
#include <IL/BGStringTemplates.h>

#include "IL/BGBase.h"

#define FILE_NUMBER FILE_NO_IL_ERROR_CPP

// Every thread has its own copy of the variable holding the last error, so be 
// sure to copy it elsewhere if you still need it after closing the thread!
IL_THREAD_VAR ILenum	lastError;


// Sets the current error
ILAPI void ILAPIENTRY il2SetError(ILenum Error) 
{
	lastError = Error;
}


//! Gets the last error
ILAPI ILenum ILAPIENTRY il2GetError(void)
{
	//SET_SCOPE; // no need here ...
	auto retVal = lastError;
	lastError = IL_NO_ERROR;
	return retVal;
}

// Set ILUT error string
void ilSetErrorA(ILimage* image, const char* errorStr)
{
	SET_SCOPE;
	bgstrlcpy(image->ErrorStr, errorStr, IL_ERROR_STRING_SIZE);
	RETURN;
}
void ilSetErrorW(ILimage* image, const wchar_t* errorStr)
{
	SET_SCOPE;
	bgstrlcpy(image->ErrorStr, errorStr, IL_ERROR_STRING_SIZE);
	RETURN;
}

void ilAddErrorA(ILimage* image, const char* errorStr)
{
	SET_SCOPE;
	bgstrlcat(image->ErrorStr, errorStr, IL_ERROR_STRING_SIZE);
	RETURN;
}
void ilAddErrorW(ILimage* image, const wchar_t* errorStr)
{
	SET_SCOPE;
	bgstrlcat(image->ErrorStr, errorStr, IL_ERROR_STRING_SIZE);
	RETURN;
}

void ilAddErrorI(ILimage* image, int64_t num, int base)
{
	SET_SCOPE;
	appendInt(image->ErrorStr, num, base, IL_ERROR_STRING_SIZE);
	RETURN;
}

const ILstring ilGetErrorString(ILimage* image)
{
	SET_SCOPE;
	ILstring result = image->ErrorStr;
	RETURN result;
}
