//-----------------------------------------------------------------------------
//
// ImageLib Sources
// Copyright (C) 2000-2009 by Denton Woods
// Last modified: 01/04/2009
//
// Filename: src-IL/src/il_files.c
//
// Description: File handling for DevIL
//
//-----------------------------------------------------------------------------


#define __FILES_C
#include "il_internal.h"
#include "IL/il2.h"
#include "PosixIO.h"
#include "LumpIO.h"
#include "Win32IO.h"
#include <stdarg.h>





// Allows you to override the default file-reading functions
/*void ILAPIENTRY iSetRead(IIO* io, fOpenProc aOpen, fCloseProc aClose, fEofProc aEof, fGetcProc aGetc, 
	fReadProc aRead, fSeekProc aSeek, fTellProc aTell)
{
	if (io != NULL) {
		memset(io, 0, sizeof(IIO));
		io->openReadOnly    = aOpen;
		io->close   = aClose;
		io->eof   = aEof;
		io->getc  = aGetc;
		io->read  = aRead;
		io->seek = aSeek;
		io->tell = aTell;
	}

	return;
}*/

// Use Posix file functions for image
/*void ILAPIENTRY il2SetReadFile(ILimage* image)
{
	if (image != NULL)
#ifdef _WIN32
		image->io = new Win32IO();
#else
		image->io = new PosixIO();
#endif
}


// Override the default file-writing functions
void ILAPIENTRY il2ResetRead(ILimage* image, IIO* newIo)
{
	if (image != NULL)
		image->io = newIo;
}*/


// Tells DevIL that we're reading from a file, not a lump
/*void iSetInputFile(ILimage* image, ILHANDLE file, ILboolean closeImage)
{
	if (image != NULL)
		image->io = new PosixIO((FILE*)file, closeImage);
}


// Tells DevIL that we're reading from a lump, not a file
void iSetInputLump(ILimage* image, void* lump, ILuint size, ILboolean deleteData)
{
	if (image != NULL) {
		image->io = new ConstLumpIO(lump, size, deleteData);
	}
}


// Tells DevIL that we're writing to a file, not a lump
void iSetOutputFile(ILimage* image, ILHANDLE file, ILboolean closeFile)
{
	if (image != NULL) {
		image->io = new PosixIO((FILE*)file, closeFile);
	}
}*/


ILuint ILAPIENTRY ilprintf(IIO* io, const char *Line, ...)
{
	char	Buffer[2048];  // Hope this is large enough
	va_list	VaLine;
	ILuint	i;

	va_start(VaLine, Line);
	vsprintf(Buffer, Line, VaLine);
	va_end(VaLine);

	i = ilCharStrLen(Buffer);
	io->write(Buffer, 1, i);

	return i;
}

// Pad zeros
void ipad(IIO* io, ILuint NumZeros)
{
	for (ILuint i = 0; i < NumZeros; i++)
		io->putc(0);
}

//
// The rest of the functions following in this file are quite
//	self-explanatory, except where commented.
//

// Next 12 functions are the default write functions

/*ILboolean ILAPIENTRY iEofFile(IIO* io)
{
	return feof((FILE*)(io->handle));
}


ILboolean ILAPIENTRY iEofLump(IIO* io)
{
	if (io->lumpSize != 0)
		return (io->rwPos >= io->lumpSize);
	return IL_FALSE;
}


ILint ILAPIENTRY iGetcLump(IIO* io)
{
	// If ReadLumpSize is 0, don't even check to see if we've gone past the bounds.
	if (io->lumpSize > 0) {
		if (io->rwPos + 1 > io->lumpSize) {
			io->rwPos--;
			il2SetError(IL_FILE_READ_ERROR);
			return IL_EOF;
		}
	}

	return *((ILubyte*)io->lump + io->rwPos++);
}


ILint64 ILAPIENTRY iReadLump(IIO* io, void *Buffer, const ILuint Size, const ILuint Number)
{
	ILint64 i, ByteSize = IL_MIN( Size*Number, io->lumpSize-io->rwPos);

	for (i = 0; i < ByteSize; i++) {
		*((ILubyte*)Buffer + i) = *((ILubyte*)io->lump + io->rwPos + i);
		if (io->lumpSize > 0) {  // ReadLumpSize is too large to care about apparently
			if (io->rwPos + i > io->lumpSize) {
				io->rwPos += i;
				if (i != Number)
					il2SetError(IL_FILE_READ_ERROR);
				return i;
			}
		}
	}

	io->rwPos += i;
	if (Size != 0)
		i /= Size;
	if (i != Number) {
		il2SetError(IL_FILE_READ_ERROR);
	}
	return i;
}


ILint64 ILAPIENTRY iSeekFile(IIO* io, ILint64 Offset, ILuint Mode)
{
	if (Mode == IL_SEEK_SET)
		Offset += io->ReadFileStart;  // This allows us to use IL_SEEK_SET in the middle of a file.
	return fseek((FILE*) io->handle, Offset, Mode);
}


// Returns 1 on error, 0 on success
ILint64 ILAPIENTRY iSeekLump(IIO* io, ILint64 Offset, ILuint Mode)
{
	switch (Mode)
	{
		case IL_SEEK_SET:
			if (Offset > (ILint)io->lumpSize)
				return 1;
			io->rwPos = Offset;
			break;

		case IL_SEEK_CUR:
			if (io->rwPos + Offset > io->lumpSize || io->rwPos+Offset < 0)
				return 1;
			io->rwPos += Offset;
			break;

		case IL_SEEK_END:
			if (Offset > 0)
				return 1;
			// Should we use >= instead?
			if (abs(Offset) > (ILint)io->lumpSize)  // If ReadLumpSize == 0, too bad
				return 1;
			io->rwPos = io->lumpSize + Offset;
			break;

		default:
			return 1;
	}

	return 0;
}


ILHANDLE ILAPIENTRY iGetFile(ILimage* image)
{
	if (image != NULL)
		return image->io.handle;
	else
		return 0;
}


const ILubyte* ILAPIENTRY iGetLump(ILimage* image) 
{
	if (image != NULL)
		return (ILubyte*)image->io.lump;
	else
		return NULL;
}


// Next 4 functions are the default write functions

ILint ILAPIENTRY iPutcFile(ILubyte Char, IIO* io)
{
	return putc(Char, (FILE*)io->handle);
}


ILint ILAPIENTRY iPutcLump(ILubyte Char, IIO* io)
{
	if (io->rwPos >= io->lumpSize || io->lump == NULL)
		return IL_EOF;

	((ILubyte*)(io->lump))[io->rwPos] = Char;
	++io->rwPos;
	return Char;
}


ILint64 ILAPIENTRY iWriteFile(const void *Buffer, ILuint Size, ILuint Number, IIO* io)
{
	ILint64 NumWritten;
	NumWritten = fwrite(Buffer, Size, Number, (FILE*) io->handle);
	if (NumWritten != Number) {
		il2SetError(IL_FILE_WRITE_ERROR);
		return 0;
	}
	return NumWritten;
}


ILint64 ILAPIENTRY iWriteLump(const void *Buffer, ILuint Size, ILuint Number, IIO* io)
{
	if (Buffer == NULL || Size == 0 || io->rwPos >= io->lumpSize || io->lump == NULL)
		return 0;

	ILubyte* inP = (ILubyte*) Buffer;
	ILubyte* outP = &((ILubyte*) io->lump)[io->rwPos];
	size_t toCopy = Size * Number;

	if (io->rwPos + toCopy > io->lumpSize)
		toCopy = io->lumpSize - io->rwPos;

	memcpy(outP, inP, toCopy);
	io->rwPos += toCopy;

	return toCopy;
}


ILuint ILAPIENTRY iTellFile(IIO* io)
{
	return ftell((FILE*) (io->handle));
}


ILint64 ILAPIENTRY iTellLump(IIO* io)
{
	return io->rwPos;
}*/

